<?php
namespace Civi\Themetest;

use CRM_Themetest_ExtensionUtil as E;
use Civi\Core\Service\AutoService;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * @service themeview
 */
class Themeview extends AutoService implements EventSubscriberInterface {

  public static function getSubscribedEvents(): array {
    return [
      '&civi.invoke.auth' => ['onInvoke', 2000],
      '&hook_civicrm_activeTheme' => ['onPickTheme', 1000],
    ];
  }

  /**
   * On every page-load, check to see if `?_tv=1`. If so, show the themeview console.
   */
  public function onInvoke($args): void {
    $tvMode = $_GET['_tv'] ?? NULL;
    unset($_GET['_tv']);

    switch ($tvMode) {
      case '1':
      case 'console':
        echo (new Console())->render();
        \CRM_Utils_System::civiExit();

      case 'isolated':
        $route = implode('/', $args);
        echo $this->invokeIsolated($route);
        \CRM_Utils_System::civiExit();
    }
  }

  /**
   * On every page-load, check to see if `?_themeView` has requested an alternate theme.
   *
   * @see \CRM_Utils_Hook::activeTheme()
   */
  public function onPickTheme(&$theme, $context): void {
    $requested = $_REQUEST['_tvTheme'] ?? NULL;
    if ($requested && $context['themes']->get($requested)) {
      $theme = $requested;
    }
  }

  public function invokeIsolated($route) {
    // \CRM_Core_Resources::singleton()->addCoreResources('html-header');

    ob_start();
    $pageContent = \CRM_Core_Invoke::invoke(explode('/', $route));
    $printedContent = ob_get_clean();
    if (empty($pageContent) && !empty($printedContent)) {
      $pageContent = $printedContent;
    }

    $htmlHeader = \CRM_Core_Region::instance('html-header')->render('', FALSE);
    $locale = \CRM_Core_I18n::getLocale();

    $output = \CRM_Core_Smarty::singleton()->fetchWith('Civi/Themetest/Isolated.tpl', [
      'lang' => substr($locale, 0, 2),
      'dir' => \CRM_Core_I18n::isLanguageRTL($locale) ? 'rtl' : 'ltr',
      'head' => $htmlHeader,
      'body' => $pageContent,
    ]);
    return $output;
  }

}
